/*******************************************************************************
Autor       : Marcel Kaptein
Datum       : 13.07.2010
Version     : 1.0
Modulname   : dip204_spi.c
Beschreibung: Dieses Modul stellt die Schnittstelle zum EA-DIP204-4 her
********************************************************************************
Version 1.0 [13.07.2010]
Neu angelegt. Der Ursprung kommt aus der Datei:lcd_dip204_spi_v21.c


--------------------------------------------------------------------------------
------------------------------------------------------------------------------*/
/****** bentigte Header- Dateien *********************************************/
#include "types.h"// fr U8
#include "dip204_spi.h"     // fr die Funktionen
//#include <util/delay.h>
#include "spi.h"

#include "projekt.h"

#include <avr/io.h>


// Variablen die fr die Instuktion des Display stehen
unsigned char power_mode_value;
unsigned char entry_mode_set_value;
unsigned char display_control_value;
unsigned char ext_function_set_value;
unsigned char cur_dis_shift_value;
unsigned char scroll_enable_value;
unsigned char function_set_value;


void wait_dip204_ready(void);



void enable_dip204(void)
{
	PORTB &= ~(1<<PB0);  /* ENABLE LCD  */
}

void disable_dip204(void)
{
	PORTB |=  (1<<PB0);
}
/*******************************************************************************
Diese Funktion erzeugt ein delay im ms- Bereich.
Input:  Zahl der ms die als Pause gewnscht ist
Output: -
*******************************************************************************/

void delay_ms(unsigned int count)
{
	while (count !=0)
	{
		delay_us(1000);
		count--;
	}
}

/* Ende der Funktion ---------------------------------------------------------*/



/*******************************************************************************
Diese Funktion sendet ein Start of Frame (SOF) an das Display.
Das SOF beinhaltet die zwei RW und RS Bits.
--------------------------------------------------------------------------------
Input:  U8 data: die Inhalte der RW und RS Bits
        Werte:   WRITE_DATA; READ_DATA; WRITE_COMMAND; READ_COMMAND
*******************************************************************************/
void send_sof(U8 data)
{
	//enable_dip204();

	send_over_spi(0x1f + data);

	//disable_dip204();
}
/* Ende der Funktion ---------------------------------------------------------*/

/*******************************************************************************
Diese Funktion sendet ein Byte an das Display. Dabei wird das Byte in zwei
Nibble zerteilt. Zuerst wird das lower Nibble gesendet mit 4 folgenden Nullen,
anschlieende das higher Nibble mit 4 folgenden Nullen.
--------------------------------------------------------------------------------
Input:  U8 mode: das zu sendende Byte
          Werte: 0 - 255
*******************************************************************************/
void send_data(U8 data)
{
	//enable_dip204();

	send_over_spi(data&0x0F);

	// jetzt muss das Nibble gedreht werden
	data = (data << 4) | ((data >> 4)&0xf);

	// high-nibble wird zu low-nibble
	// jetzt kann man wieder das lower Nibble senden
	send_over_spi(data&0x0F);

	//disable_dip204();
}
/* Ende der Funktion ---------------------------------------------------------*/

// zusammenhngende Daten die gesendet werden sollen nicht unterbrochen werden
// laut ks0073 Datenbblatt sollen nach drei Bytes kurz gewartet werden
// SOF und dann drei Bytes
// z.B. send_sof()und dann send_data()
void wait_dip204_ready(void)
{

	volatile unsigned char temp = 0;

	send_sof(READ_COMMAND);

	do
	{
		//enable_dip204();
		send_over_spi(0x00);
		temp = spi_getchar();
		//disable_dip204();

	}while(temp & 0x80 );


}
/*******************************************************************************
Diese Funktion manipuliert fr smtliche Funktionen die gewnsten bits
in den bergebenen Register
Input:
U8 *reg:	bergebenes Registe in dem ein bit manipuliert werden soll
U8 *bit:	welches bit im Register manipuliert werden soll
U8 *value	wert den das bit haben soll
 *******************************************************************************/
void set_register_values(unsigned char *reg, unsigned char *bit, unsigned char *value)
{
	if(*(value))			// wenn das bit "1" werden soll...
	{
		*(reg)|= *(bit);
	}

	else					// ...oder das bit 0 werden soll
	{
		*(reg) &= ~*(bit);
	}
}
/* Ende der Funktion ---------------------------------------------------------*/

/*******************************************************************************
Diese Funktion lscht das Display
*******************************************************************************/
void lcd_clear(void)
{
	send_sof(WRITE_COMMAND);
	send_data(CLEAR_DISPLAY);
	delay_ms(4);
}
/* Ende der Funktion ---------------------------------------------------------*/

/*******************************************************************************
Diese Funktion setzt den Cursor @ Home
*******************************************************************************/
void cursor_home(void)
{
	send_sof(WRITE_COMMAND);
	send_data(CURSOR_HOME);
	delay_ms(4);
}
/* Ende der Funktion ---------------------------------------------------------*/

/*******************************************************************************
Diese Funktion setzt den Power Mode des Displays
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void power_mode(unsigned char bit, unsigned char value)
{
	set_register_values(&power_mode_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(RE_ON);   // RE Bit auf 1
	send_data(power_mode_value);
	send_data(RE_OFF);  // RE Bit wieder auf 0
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion setzt den Entry Mode des Displays
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void entry_mode(unsigned char bit, unsigned char value)
{
	set_register_values(&entry_mode_set_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(entry_mode_set_value);
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion steuert die Displayfunktionen
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void display_control(unsigned char bit, unsigned char value)
{
	set_register_values(&display_control_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(display_control_value);
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion setzt die extended Funktionen des Displays
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void ex_function_set(unsigned char bit, unsigned char value)
{
	set_register_values(&ext_function_set_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(RE_ON);   // RE Bit auf 1
	send_data(ext_function_set_value);
	send_data(RE_OFF);  // RE Bit wieder auf 0
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion legt fest ob sich der Cursor oder das Display bewegt
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void cur_dis_shift(unsigned char bit, unsigned char value)
{
	set_register_values(&cur_dis_shift_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(cur_dis_shift_value);
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion schaltet das scrollen einer Zeile ein
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void scroll_enable(unsigned char bit, unsigned char value)
{
	set_register_values(&scroll_enable_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(RE_ON);   // RE Bit auf 1
	send_data(scroll_enable_value);
	send_data(RE_OFF);  // RE Bit wieder auf 0
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion setzt smtliche Funktionen des Displays
--------------------------------------------------------------------------------
Input:
U8 bit:		welches bit manipuliert werden soll
U8 value: 	Wert des ausgewhlten bits
*******************************************************************************/
void function_set(unsigned char bit, unsigned char value)
{
	set_register_values(&function_set_value, &bit, &value);
	send_sof(WRITE_COMMAND);
	send_data(function_set_value);
}
/* Ende der Funktion ---------------------------------------------------------*/



void dip204_invert(unsigned char value)
{
	function_set(REVERS_BIT, value);
}

/*******************************************************************************
Diese Funktion schreibt ein Zeichen auf das Display
--------------------------------------------------------------------------------
Input:  U8 data: das zu sendende Zeichen
          Werte: 0-255
*******************************************************************************/
void dip204_putchar(U8 data)
{
	send_sof(WRITE_DATA);
	send_data(data);
}

/* Ende der Funktion ---------------------------------------------------------*/

/*******************************************************************************
Diese Funktion sendet ein Textstring an das Display.
--------------------------------------------------------------------------------
Input:
U8 *string: das zu sendende Textstring an das Display
*******************************************************************************/
void dip204_putstring(unsigned char *data)
{
	send_sof(WRITE_DATA);

	while(*data)
	{
		if(*data == '')*data = 0x5B;
		if(*data == '')*data = 0x5C;
		if(*data == '')*data = 0x5E;
		if(*data == '')*data = 0x7B;
		if(*data == '')*data = 0x7C;
		if(*data == '')*data = 0x7E;
		send_data(*data++);
	}
}
/* Ende der Funktion ---------------------------------------------------------*/

/*******************************************************************************
Diese Funktion setzt der Cursor an eine bestimmte Position auf dem Display.
Die kleinste Position ist Zeile 0, Position 0.
Die grte Position ist Zeile 3, Position 19.
Der Cursor wird positioniert, indem man das DDRAM Register beschreibt. Dies wird
mit einem Kommando und dem Startwert 0x80 begonnen. Zu diesem Startwert wird
dann die Position dazuaddiert. Ab der Adresse 0x20, 0x40 und 0x60 beginnt
jeweils eine neue Zeile.
--------------------------------------------------------------------------------
Input:  U8 x: X- Position der gewnschten Position
        U8 y: Y- Position der gewnschten Position

*******************************************************************************/
void dip204_gotoxy(U8 column, U8 line)
{
	send_sof(WRITE_COMMAND);
	send_data(0x80 + column + (line<<5) );
}





/*******************************************************************************
Diese Funktion initialisiert das EA-DIP204
--------------------------------------------------------------------------------
*******************************************************************************/
void init_dip204(void)
{
	// die Variablen werden hier definiert damit man die Werte auch im
	// laufenden betrieb zurcksetzen kann
	power_mode_value = 0x02;
	entry_mode_set_value = 0x04;
	display_control_value = 0x08;
	ext_function_set_value = 0x08;
	cur_dis_shift_value = 0x10;
	scroll_enable_value = 0x10;
	function_set_value = 0x20;

	init_spi4dip204();
	DDRB  |=   (1<<PB0);              // EN Ausgang

	enable_dip204();
	delay_us(1000);
	ex_function_set(LINE,LINE_4);
	ex_function_set(CURSOR,NORMAL);
	display_control(DISPLAY,ON);
	lcd_clear();
	cursor_home();
	entry_mode(CURSOR,INCREMENT);

}
/* Ende der Funktion ---------------------------------------------------------*/
