#ifndef DIP_204_SPI_H
#define DIP_204_SPI_H

#include "types.h"
#include "spi.h"

/*
- SCLK muss im Ruhezustand HIGH sein  -> MODE_2 oder MODE_3

- Daten werde eingelesen wenn SCLK
  von LOW nach HIGH wechselt          -> MODE_3

Laut Datenblatt vom KS0073 werden die zu sendende Daten in zwei
Nibble bertragen. Es wird erst das lower Nible (D0-D3) und
dann das higher Nibble (D4-D7) bertragen.
Damit man dann nicht fr alle Kommandos die defines "umdrehen"
muss, wird die bertragungsart "LSB first" verwendet!

- LSB first

Das Display sendet LSB- first. Somit erkennt man auf dem Oszi auch
von links nach rechts D0...D7
*/

//#define enable_dip204()  enable_separate_ss();
//#define disable_dip204() disable_separate_ss();
#define send_over_spi(data) spi_putchar(data);
#define init_spi4dip204() init_spi();
/*
#define enable_dip204()        PORTB &= ~(1<<PB0)  // ENABLE LCD  /
#define disable_dip204()        PORTB |=  (1<<PB0)
*/

#define XTAL 8000000	// Definition der Taktfrequenz. Ist fr die Funktion "delay" wichtig


/*******************************************************************************
Diese Funktion erzeugt ein delay von lediglich 4 Taktzyklen
Input:  -
Output: -
*******************************************************************************/

static inline void _delayFourCycles(unsigned int __count)
{
    if ( __count == 0 )
        __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"
    	    "brne 1b"                              // 4 cycles/loop
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}

/* Ende der Funktion ---------------------------------------------------------*/




/*******************************************************************************
Diese Definition erzeugt ein delay von wenigen s. Wichtig hierbei ist, die
richtige Definition der Taktfrequenz mit dem der Controller betrieben wird.
Diese Definition muss in der .h Datei unter [XTAL] eingestellt werden.
Input:  Zahl der s die als Pause gewnscht ist
Output: -
*******************************************************************************/
#define delay_us(us)  _delayFourCycles( ( ( 1*(XTAL/4000) )*us)/1000 )
/* Ende der Definition -------------------------------------------------------*/



/*******************************************************************************
Definitionen fr das Programm
*******************************************************************************/
// Definition fr das RE- Bit im "Function Set" Register
#define RE_ON         0x34
#define RE_OFF        0x30

// Achtung, bei den folgenden Definitionen gilt LSB- first !!
#define WRITE_COMMAND 0x00 // RW = 0; RS = 0
#define READ_COMMAND  0x20 // RW = 1; RS = 0
#define WRITE_DATA    0x40 // RW = 0; RS = 1
#define READ_DATA     0x60 // RW = 1; RS = 1
#define READ_BUSY_FLAG 		  0xC0
#define BUSY_FLAG		0x40

/* Funktion >clear_display< */
#define CLEAR_DISPLAY 	0x01

/* Funktion >cursor_home< */
#define CURSOR_HOME   	0x02

/* Funktion >power_mode< */
#define POWERDOWN		0x01


/* Funktion >entry_mode< */
#define CURSOR     		0x02
#define DECREMENT    	0
#define INCREMENT    	1

/* Funktion >display_control< */
#define BLINK      		0x01
// CURSOR ist schon definiert
#define DISPLAY			0x04

/* Funktion >ex_function_set< */
#define LINE			0x01
// CURSOR ist schon definiert
#define DOT				0x04
#define LINE_2        	0
#define LINE_4        	1
#define NORMAL   		0
#define INVERTING    	1
#define DOT5          	0
#define DOT6          	1

/* Funktion >cur_dis_shift< */
#define DIRECTION		0x04
#define SHIFT			0x08
#define SHIFT_LEFT		0
#define SHIFT_RIGHT		1
#define SHIFT_CURSOR	0
#define SHIFT_DISPLAY	1

/* Funktion >scroll_enable< */
#define SCROLL			0x10	// hier gibt es keine "Auswahlbit"
#define SCROLL_LINE0	1
#define SCROLL_LINE1	2
#define SCROLL_LINE2	4
#define SCROLL_LINE3	8



/* Funktion >function_set< */
#define REVERS_BIT		0x01
#define SCROLL_SHIFT	0x02
#define DATA_LENGTH		0x10
// NORMAL ist schon definiert
// INVERTING ist schon definiert
#define SCROLL_DOT		0
#define SHIFT_DISPLAY	1


extern void dip204_gotoxy(U8, U8);
extern void dip204_putchar(U8);
extern void dip204_putstring(unsigned char *string);
extern void init_dip204(void);

#endif  // LCD_DIP_204_SPI_H






