/*******************************************************************************
Autor       : Marcel Kaptein
Datum       :
Version     :
Modulname   : projekt.h
Beschreibung: Diese Header-Datei ist bei jedem Projekt neu zu erstellen
********************************************************************************
------------------------------------------------------------------------------*/
#ifndef _PROJEKT_H_
#define _PROJEKT_H_

/****** bentigte Header- Dateien *********************************************/
#include "types.h"



/***** Definitionen ***********************************************************/
#ifndef F_CPU
#define F_CPU 8000000UL  // Systemtakt in Hz - Definition als unsigned long beachten
                         // Ohne ergeben sich unten Fehler in der Berechnung
#endif


#define PROZESSOR_FREQUENZ 2*4000000

//#define USE_UART
#define USE_SPI
//#define USE_TIMER_0
//#define USE_TIMER_1
//#define USE_TIMER_2


#ifdef USE_UART
//--- Definitionen fr UART ---//
	#define NONE					0
	#define EVEN					2
	#define ODD						3
	#define ONE_BIT					0
	#define TWO_BIT					1
	#define FIVE_BIT				0
	#define SIX_BIT  				1
	#define SEVEN_BIT  				2
	#define EIGHT_BIT 				3
	#define NINE_BIT 				7

	//#define BAUDRATE 9600
	#define BAUDRATE 				19200
	#define PARITY 					NONE
	#define STOP_BIT 				ONE_BIT
	#define DATA_SIZE 				EIGHT_BIT

	#define SEND_UART_OK		// Ob nach der Initialisierung "UART_OK" gesendet werden  soll
#endif	// ENDE: #ifdef UART


//--- Definitionen fr SPI ---//
#ifdef USE_SPI
	#define LSB						1
	#define MSB 					0
	#define MASTER 					1
	#define SLAVE 					0
	#define MODE_0 					0	// Idle low; senden von "no idle" to "idle"
	#define MODE_1 					1	// Idle low; senden von "idle" to "no idle"
	#define MODE_2 					2	// Idle high; senden von "no idle" to "idle"
	#define MODE_3 					3	// Idle high; senden von "idle" to "no idle"

										//bei 8MHz
	#define FOSC_2 					4	//    4MHz
	#define FOSC_4 					0	//    2MHz
	#define FOSC_8 					5	//    1MHZ
	#define FOSC_16 				1	//  500kHz
	#define FOSC_32 				6	//  250kHz
	#define FOSC_64 				2	//  125kHz
	#define FOSC_128 				3	// 62,5kHz

// ob erst MSB oder LSB bertragen werden soll
	#define DATA_TRANSMIT_FIRST 	LSB

// ob der Controller als SPI- Master oder Slave funktionieren soll
	#define SPI_FUNCTION 			MASTER

// welcher SPI- Mode fr die bertragung verwendet werden soll
	#define SPI_MODE 				MODE_3

// welcher SPI- Takt gewhlt werden soll
	#define SPI_CLOCK_RATE 			FOSC_64

// ob der SPI- Interrupt verwendet werden soll
	//#define US_SPI_INTERRUP

// wenn der Controller als SPI- Master funktioniert und
// der Slave nicht am normalen /SS- Pin angeschlossen ist
	//#define SEPARATE_SS_PIN_IN_USE

	#if (SPI_FUNCTION == MASTER)
		#ifdef SEPARATE_SS_PIN_IN_USE
			#define SEPARATE_SS_DDR			DDRB
			#define SEPARATE_SS_PORT		PORTB
			#define SEPARATE_SS_PIN			PB0
		#endif
	#endif

#endif // ENDE: #ifdef SPI


//--- Definitionen fr den Timer 0 ---//
#ifdef USE_TIMER_0
// F_CPU = 8MHz
//	PS		T(Interrupt)
//	0		25,5s
//	8		255s
//	64		2ms
//	256		8ms
//	1024	32,4ms
#define TIMER0_PRESCALE_0		1
#define TIMER0_PRESCALE_8		2
#define TIMER0_PRESCALE_64		3
#define TIMER0_PRESCALE_256		4
#define TIMER0_PRESCALE_1024	5

// Definiert mit welchem Takt der Timer0 laufen soll
#define TIMER0_PRESCALE TIMER0_PRESCALE_8

// Definiert ob der Timer0 als Timer- Maschine laufen soll
#define TIMER_0_IS_TIME_MACHINE
// [TIME_MACHINE_DIVISOR] wird in der [timer.h] definiert.

#endif // ENDE: #ifdef USE_TIMER_0


/****** Typ deklarationen *****************************************************/

/****** externe Variablen deklarationen ***************************************/

/****** externe Funktions deklarationen ***************************************/



#endif  // PROJEKT_H
