/*******************************************************************************
Autor       : Marcel Kaptein
Datum       : 30.06.2010
Version     : 1.0
Modulname   : spi.c
Beschreibung: Dieses Modul stellt die Schnittstelle zum SPI her.
********************************************************************************
Version 1.0 [30.06.2010]
Neu angelegt.


--------------------------------------------------------------------------------
------------------------------------------------------------------------------*/
/****** bentigte Header- Dateien *********************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "spi.h"
#include "types.h"
#include "projekt.h"
/***** Definitionen ***********************************************************/


/****** Typ deklarationen *****************************************************/


/****** Variablen deklarationen ***********************************************/


/****** Funktions deklarationen ***********************************************/


/*******************************************************************************
Diese Funktion initialisiert die SPI- Schnittstelle im Controller.
Alle wichtigen Initialisierungen mssen in der [projekt.h] gettigt werden.
*******************************************************************************/
void init_spi(void)
{
	set_data_order();			// LSB / MSB fist
	set_spi_function();			// ob Master oder Slave- Betrieb
	set_spi_mode();				// setzen des SPI-Modes (Mode_0 - Mode_3)
	set_spi_clock_rate();		// SPI- Takt setzen
	enable_spi();				// SPI einschalten

#ifdef US_SPI_INTERRUP			// wenn SPI- Interrupt gewnscht wird
	enable_spi_interrupt();
#endif

#ifdef SEPARATE_SS_PIN_IN_USE	// Wenn der SPI- Slave Select nicht
	set_separate_slave_select();// am normalen /SS- Pin angeschlossen ist
#endif
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion sendet ein Byte ber das SPI Datenregister
Input:  U8: die zu sendende Daten
 *******************************************************************************/
void spi_putchar(U8 data)
{
	spi_data_write(data);
	while(spi_busy());
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion sendet ein DUMMY-Byte ber SPI
 *******************************************************************************/
void spi_send_dummy(void)
{
	spi_data_write(0xFF);
	while(spi_busy());
}
/* Ende der Funktion ---------------------------------------------------------*/


/*******************************************************************************
Diese Funktion liest ein empfangenes Byte aus dem SPI- Datenregister
Return:  U8: das empfangende Byte
 *******************************************************************************/
U8 spi_getchar( void )
{
	while(spi_busy());
	return spi_data_read();
}
/* Ende der Funktion ---------------------------------------------------------*/



#ifdef US_SPI_INTERRUP
ISR (SPI_INTERRUPT_VECTOR)
{
   //irgendein Code
}
#endif
