/*******************************************************************************
Autor       : Marcel Kaptein
Datum       : 30.06.2010
Version     : 1.0
Modulname   : spi.h
Beschreibung: Diese Header-Datei beschreibt die UART
********************************************************************************
Version 1.0 [30.06.2010]
Neu angelegt.


--------------------------------------------------------------------------------
------------------------------------------------------------------------------*/
#ifndef _SPI_H_
#define _SPI_H_


//_____  I N C L U D E S _______________________________________________________

/****** bentigte Header- Dateien *********************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "types.h"
#include "projekt.h"

/***** Definitionen ***********************************************************/
//>>>>>> ------ ATMEGA644P ------ <<<<<< //
#if defined(__AVR_ATmega644P__)
#include "spi/spi_mega644.h"

//>>>>>> ------ ATMEGA16 ------ <<<<<< //
#elif defined(__AVR_ATmega16__)
#include "spi/spi_mega644.h"


//>>>>>> ------ ATMEGA128 ------ <<<<<< //
#elif defined(__AVR_ATmega128__)

	// Festlegung der SPI- Pins
	#define PORT_SPI PORTB
	#define PIN_SPI PINB
	#define DDR_SPI DDRB
	#define SS        PB0
	#define DDR_SS    DDB0
	#define MOSI      PB2
	#define DDR_MOSI  DDB2
	#define MISO      PB3
	#define DDR_MISO  DDB3
	#define SCK       PB1
	#define DDR_SCK   DDB1

	// SPI Data Register
	#define DATAREG SPDR
	// SPI State Register
	#define STATEREG SPSR
	// SPI Interrupt Flag
	#define SPIINTFLAG SPIF

//>>>>>> ------ ATtiny85 ------ <<<<<< //
#elif defined (__AVR_ATtiny85__)

	// Festlegung der SPI- Pins
	#define PORT_SPI PORTB
	#define PIN_SPI PINB
	#define DDR_SPI DDRB
	#define PORT_SPI PORTB
	#define PIN_SPI PINB
	#define DDR_SPI DDRB

	#define SS        PB4	// Dieser Pin soll einen zustzlichen SS- Pin
	#define DDR_SS    DDB4	// eines Slaves ansteuern knnen
	#define MOSI      PB0
	#define DDR_MOSI  DDB0
	#define MISO      PB1
	#define DDR_MISO  DDB1
	#define SCK       PB2
	#define DDR_SCK   DDB2

	// SPI Data Register
	#define DATAREG USIDR
	// SPI State Register
	#define STATEREG SPSR
	// SPI Interrupt Flag
	#define SPIINTFLAG SPIF

#else
#    warning "Fr diesen Controller gibt es keine Definitionen"
#endif

#ifdef SEPARATE_SS_PIN_IN_USE
#define set_separate_slave_select() (SEPARATE_SS_DDR  |= (1<<SEPARATE_SS_PIN) )	// Special Pin Select als Ausgang
#define disable_separate_ss() 		(SEPARATE_SS_PORT |=  (1<<SEPARATE_SS_PIN) )
#define enable_separate_ss() 		(SEPARATE_SS_PORT &= ~(1<<SEPARATE_SS_PIN) )


#endif

// SPI Control Register
#define disable_spi_interrupt() 	(SPI_CONT_REG  &= ~(1<<SPIE) )
#define enable_spi_interrupt() 		(SPI_CONT_REG  |=  (1<<SPIE) )
#define disable_spi() 				(SPI_CONT_REG  &= ~(1<<SPE) )
#define enable_spi() 				(SPI_CONT_REG  |=  (1<<SPE) )
#define deactivate_slave()			(SPI_PORT      |=  (1<<SS) )
#define activate_slave()			(SPI_PORT      &= ~(1<<SS) )
/*
#define pins_4_master()				{SPI_DDR       |=  (1<<DDR_SS);\
									 SPI_DDR       |=  (1<<DDR_MOSI);\
									 SPI_DDR       |=  (1<<DDR_SCK);}
*/
#define pins_4_master()				{SPI_DDR       |=  (1<<DDR_SS);\
									 SPI_DDR       |=  (1<<DDR_MOSI);\
									 SPI_DDR       |=  (1<<DDR_SCK);\
									 SPI_PORT	   |=  (1<<MISO);}

#define pins_4_slave()				{SPI_DDR       |=  (1<<DDR_MISO);}
#define set_data_order() 			(SPI_CONT_REG  |=  (DATA_TRANSMIT_FIRST<<DORD) )
#define set_spi_mode() 				(SPI_CONT_REG  |=  (SPI_MODE<<CPHA) )
#define set_spi_clock_rate() 		{SPI_CONT_REG  |=  ( (SPI_CLOCK_RATE & 3) << SPR0);\
									 SPI_STATE_REG |=  ( (SPI_CLOCK_RATE >> 2) << SPI2X);}
// laut Datenblatt muss der SS-Pin vor dem setzen in den Master-Mode deaktiviert
// werden sonnst wird das Master-Bit wieder zurckgesetzt
#if (SPI_FUNCTION == MASTER)
#define set_spi_function() 			{deactivate_slave();\
									pins_4_master();\
									(SPI_CONT_REG |=  (SPI_FUNCTION<<MSTR) );}
#elif (SPI_FUNCTION == SLAVE)
#define set_spi_function() 			{pins_4_slave();\
									(SPI_CONT_REG |=  (SPI_FUNCTION<<MSTR) );}
#endif

#define spi_busy() 					(!(SPI_STATE_REG & (1<<SPI_INT_FLAG)))
#define spi_data_write(data) 		(SPI_DATA_REG = data)
#define spi_data_read()  			SPI_DATA_REG

#define disable_spi_slave() 		(SPI_PORT &= ~(1<<SS))
#define enable_spi_slave() 			(SPI_PORT |= (1<<SS))


/*
Mode_0:
Der Bus ist im Idle- Zustand low
Die Daten werden gesendet wenn die Flanke von high nach low wecheselt.
senden von "no Idle" to "idle"
-> Idle -> steigende Flanke -> falende Flanke = senden

senden     |       |
       |---|   |---|
       |   |   |   |
Idel---|   |---|   |---|
Start  |       |

Mode_1:
Der Bus ist im Idle- Zustand low
Die Daten werden gesendet wenn die Flanke von low nach high wecheselt.
senden von "Idle" to "no idle"
-> Idle -> steigende Flanke = senden

senden     |       |
       |   |---|   |---|
       |   |   |   |
Idel---|---|   |---|
Start  |       |

Mode_2:
Der Bus ist im Idle- Zustand high
Die Daten werden gesendet wenn die Flanke von low nach high wecheselt.
senden von "no Idle" to "idle"
-> Idle -> falende Flanke -> steigende Flanke = senden

senden     |       |
Idel---|   |---|   |
       |   |   |   |
       |---|   |---|---|
Start  |       |

Mode_3:
Der Bus ist im Idle- Zustand high
Die Daten werden gesendet wenn die Flanke von high nach low wecheselt.
senden von "Idle" to "no idle"
-> Idle -> falende Flanke -> steigende Flanke = senden

senden     |       |
Idel---|---|   |---|
       |   |   |   |
       |   |---|   |---|
Start  |       |
*/


/****** Typ deklarationen *****************************************************/

/****** externe Variablen deklarationen ***************************************/

/****** externe Funktions deklarationen ***************************************/

extern void init_spi(void);
extern void spi_putchar(unsigned char);
extern void spi_send_dummy(void);
extern unsigned char spi_getchar(void);
extern unsigned char spi_getchar2(void);

#endif /* _SPI_V03_H_ */
